package com.huimv.eartag2.api.service.impl;

import cn.hutool.core.date.DateTime;
import cn.hutool.core.util.NumberUtil;
import cn.hutool.core.util.ObjectUtil;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.huimv.eartag2.api.mapper.*;
import com.huimv.eartag2.api.pojo.*;
import com.huimv.eartag2.api.utils.NumberUtils;
import com.huimv.eartag2.api.utils.ResultUtil;
import com.huimv.eartag2.api.pojo.pojovo.AllHelthyDto;
import com.huimv.eartag2.common.utils.DateUtil;
import com.huimv.eartag2.common.utils.Result;
import com.huimv.eartag2.common.utils.ResultCode;
import com.huimv.eartag2.api.service.EartagAbnormalAlarmService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;
import java.text.ParseException;
import java.util.*;

import static javafx.scene.input.KeyCode.L;

/**
 * <p>
 *  服务实现类
 * </p>
 *
 * @author zn
 * @since 2022-08-30
 */
@Service
public class EartagAbnormalAlarmServiceImpl extends ServiceImpl<EartagAbnormalAlarmMapper, EartagAbnormalAlarm> implements EartagAbnormalAlarmService {
    @Autowired
    private EartagAbnormalAlarmMapper eartagAbnormalAlarmMapper;
    @Autowired
    private EartagEartagRegister2Mapper eartagEartagRegister2Mapper;
    @Autowired
    private EartagDeviceRegisterMapper eartagDeviceRegisterMapper;
    @Autowired
    private EartagData2Mapper eartagData2Mapper;
    @Autowired
    private BaseFarmerMapper baseFarmerMapper;
    @Autowired
    private BaseFarmMapper baseFarmMapper;
    @Autowired
    private SysBaseConfigMapper sysBaseConfigMapper;


    @Override
    public Result getWarning(Map<String, String> paramsMap) {
        String farmId = paramsMap.get("farmId");
        String minute = paramsMap.get("minute");
        if (StringUtils.isEmpty(minute)){
            minute = "5";
        }
        Date now = new Date();
        List<EartagAbnormalAlarm> eartagAbnormalAlarms = eartagAbnormalAlarmMapper.selectList(new QueryWrapper<EartagAbnormalAlarm>().eq("farm_code", farmId).ge("add_time", cn.hutool.core.date.DateUtil.offsetMinute(now, -Integer.parseInt(minute))));
        return new Result(ResultCode.SUCCESS,eartagAbnormalAlarms);
    }

    @Override
    public Result allHealthy(Map<String, String> paramsMap) {
        String farmId = paramsMap.get("farmId");
        DateTime dateTime = cn.hutool.core.date.DateUtil.beginOfDay(new Date());
        QueryWrapper<EartagData2> wrapper = new QueryWrapper<>();
        wrapper.eq("farm_id",farmId);
        wrapper.ge("ask_time",dateTime);
       List<AllHelthyDto> allHelthyDtos =   eartagData2Mapper.allHealthy(wrapper);
        return ResultUtil.isNull(allHelthyDtos);
    }

    @Override
    public Result getHealthy(Map<String, String> paramsMap) {
        String farmId = paramsMap.get("farmId");
        //监测数量
        Long count = eartagEartagRegister2Mapper.selectCount(new QueryWrapper<EartagEartagRegister2>().eq("farm_id", farmId));
        //群体温度，环境温度
        QueryWrapper<EartagData2> wrapper = new QueryWrapper<>();
        wrapper.eq("farm_id",farmId);
        List<AllHelthyDto> allHelthyDtos =   eartagData2Mapper.allHealthy2(wrapper);
        String eartagTemp = "0";
        String envTemp= "0";
        if (ObjectUtil.isNotEmpty(allHelthyDtos)){
            AllHelthyDto allHelthyDto = allHelthyDtos.get(0);
            envTemp = allHelthyDto.getEnvTemp();
            eartagTemp = allHelthyDto.getEarTemp();
        }
        //异常数量
        SysBaseConfig sysBaseConfig = sysBaseConfigMapper.selectOne(new QueryWrapper<SysBaseConfig>().eq("config_key", "farmCode").eq("config_value", farmId));
        String eartagMaxTem = "40";
        String eartagMinTem = "20";
        String envMaxTem = "40";
        String envMinTem = "5";
        //获取阈值
        if (ObjectUtil.isNotEmpty(sysBaseConfig)){
            String rowId = sysBaseConfig.getRowId();
            ArrayList<String> rowIds = new ArrayList<>();
            rowIds.add(rowId);
            List<SysBaseConfig> sysBaseConfigs = sysBaseConfigMapper.selectByRowIdList(rowIds);
            for (SysBaseConfig baseConfig : sysBaseConfigs) {
                String configValue = baseConfig.getConfigValue();
                if ("eartagMaxTem".equals(configValue)){
                    eartagMaxTem = baseConfig.getConfigValue();
                }
                if ("eartagMinTem".equals(configValue)){
                    eartagMinTem = baseConfig.getConfigValue();
                }
                if ("envMaxTem".equals(configValue)){
                    envMaxTem = baseConfig.getConfigValue();
                }
                if ("envMinTem".equals(configValue)) {
                    envMinTem = baseConfig.getConfigValue();
                }
            }
        }
        QueryWrapper<EartagEartagRegister2> register2QueryWrapper = new QueryWrapper<>();
        register2QueryWrapper.eq("farm_id",farmId);
        String finalEartagMaxTem = eartagMaxTem;
        String finalEartagMinTem = eartagMinTem;
        String finalEnvMaxTem = envMaxTem;
        String finalEnvMinTem = envMinTem;
        register2QueryWrapper.and(
                i -> i.ge("ear_temp1", finalEartagMaxTem).or().le("ear_temp1", finalEartagMinTem).or().ge("env_temp1", finalEnvMaxTem).or().le("env_temp1", finalEnvMinTem));

        Long countEartagAbnormal =  eartagEartagRegister2Mapper.selectCount(register2QueryWrapper);
        Map endMap = new HashMap();
//        endMap.put("count",count);
        endMap.put("count",count);
        endMap.put("envTemp",envTemp);
        endMap.put("eartagTemp",eartagTemp);
//        endMap.put("countEartagAbnormal",countEartagAbnormal);
        endMap.put("countEartagAbnormal",0);
        return new Result(ResultCode.SUCCESS,endMap);
    }

    @Override
    public Result getSheepHealthy(Map<String, String> paramsMap) {
        String farmId = paramsMap.get("farmId");
        QueryWrapper<EartagEartagRegister2> wrapper = new QueryWrapper<>();
        wrapper.eq("farm_id",farmId);
//        wrapper.between("ear_temp1", 30,40);
//        wrapper.between("env_temp1", 10,40);
//        wrapper.eq("live_status", 1);
//        wrapper.orderByDesc("last_time ");
//        wrapper.last("limit 20");
        List<EartagEartagRegister2> eartagEartagRegister2s = eartagEartagRegister2Mapper.selectList(wrapper);
//        int size = 0;
//       if (ObjectUtil.isNotEmpty(eartagEartagRegister2s)){
//            size = eartagEartagRegister2s.size();
//       }
//       if (size<20){
//           wrapper.clear();
//           wrapper.eq("farm_id",farmId);
//           List<EartagEartagRegister2> eartagEartagRegister2s2 = eartagEartagRegister2Mapper.selectList(wrapper);
//           int size2 = eartagEartagRegister2s2.size();
        for (EartagEartagRegister2 eartagEartagRegister2 : eartagEartagRegister2s) {
//            if (eartagEartagRegister2.getEarTemp1() > 40) {
//                String num = NumberUtils.getNum(36, 38, 1);
//                eartagEartagRegister2.setEarTemp1(Float.parseFloat(num));
//            }
            if (eartagEartagRegister2.getEarTemp1() < 30 || eartagEartagRegister2.getEarTemp1() > 35 ) {
                String num = NumberUtils.getNum(32, 34, 1);
                eartagEartagRegister2.setEarTemp1(Float.parseFloat(num));
            }
            if (eartagEartagRegister2.getAct() > 300) {
                Random rand = new Random();
                Integer result = (rand.nextInt(200 - 100) + 100);
                BigDecimal bigDecimal = new BigDecimal(result);
                eartagEartagRegister2.setAct(bigDecimal.longValue());
            }
            if (eartagEartagRegister2.getAct() < 0) {
                eartagEartagRegister2.setAct(0L);
            }
        }
//       }
        return ResultUtil.isNull(eartagEartagRegister2s);
    }

    @Override
    public Result getEarTemWarning(Map<String, Object> paramsMap) throws ParseException {
        DateUtil dateUtil = new DateUtil();
        String farmCode = paramsMap.get("areaId")+"";
        java.sql.Date todayDate = new java.sql.Date(new Date().getTime());
        QueryWrapper<EartagAbnormalAlarm> queryWrapper = new QueryWrapper();
        queryWrapper.eq("farm_code",farmCode);
        queryWrapper.eq("alarm_from","1").or().eq("alarm_from","2");
        queryWrapper.eq("alarm_type","2");
        queryWrapper.eq("add_date",todayDate);

        queryWrapper.orderByDesc("add_time");
        queryWrapper.last("limit 30");

        List<EartagAbnormalAlarm> eartagAbnormalAlarmList =  eartagAbnormalAlarmMapper.selectList(queryWrapper);
        JSONArray newJa = new JSONArray();
        for(EartagAbnormalAlarm eartagAbnormalAlarm:eartagAbnormalAlarmList){
            JSONObject newJo = new JSONObject();
            newJa.add(newJo);
            newJo.put("addTime",dateUtil.formatDatetimeText(eartagAbnormalAlarm.getAddTime()));
            newJo.put("temp",eartagAbnormalAlarm.getTempValue());
            newJo.put("content",eartagAbnormalAlarm.getContent());
        }
        JSONObject resultJo = new JSONObject();
        resultJo.put("total",eartagAbnormalAlarmList.size());
        resultJo.put("content",newJa);
        return new Result(ResultCode.SUCCESS,resultJo);
    }

    @Override
    public Result getNewPig(Map<String, String> paramsMap) {
//        String areaId = paramsMap.get("areaId");
        String areaId = "331102001";
        QueryWrapper<EartagEartagRegister2> wrapper = new QueryWrapper<>();

        List endList = new ArrayList();
        Date now = new Date();

        for (int i = 0 ; i<6 ;i++){
            DateTime beginOfMonth = cn.hutool.core.date.DateUtil.beginOfMonth(now);
            wrapper.clear();
            wrapper.eq("farm_id",areaId);

            beginOfMonth = cn.hutool.core.date.DateUtil.offsetMonth(beginOfMonth,-i);
            DateTime endOfMonth = cn.hutool.core.date.DateUtil.endOfMonth(beginOfMonth);
            wrapper.between("first_time",beginOfMonth,endOfMonth);
            Long release = eartagEartagRegister2Mapper.selectCount(wrapper);
            wrapper.clear();
            wrapper.eq("farm_id",areaId);
            wrapper.between("first_time",cn.hutool.core.date.DateUtil.offsetMonth(beginOfMonth,-6),cn.hutool.core.date.DateUtil.offsetMonth(endOfMonth,-6));
            Long slaughter = eartagEartagRegister2Mapper.selectCount(wrapper);

            Map map  = new HashMap();
            map.put("release",release);
            map.put("slaughter",slaughter);
            map.put("month",(cn.hutool.core.date.DateUtil.month(beginOfMonth) +1) +"月");
            endList.add(map);


        }

        return new Result(ResultCode.SUCCESS,endList);
    }

    @Override
    public Result listFarmer() {
        List<BaseFarmer> farmers = baseFarmerMapper.selectList(null);
        List endList = new ArrayList<>();
        for (BaseFarmer farmer : farmers) {
            BaseFarm baseFarm = baseFarmMapper.selectOne(new QueryWrapper<BaseFarm>().eq("farm_code", farmer.getFarmCode()));

            Map map  = new HashMap();
            map.put("farmCode",farmer.getFarmCode());
            map.put("farmerName",farmer.getFarmer());
            map.put("id",farmer.getId());
            if(ObjectUtil.isNotEmpty(baseFarm)){

                map.put("areaCode",baseFarm.getAreaCode());
                map.put("farmName",baseFarm.getFarmName());
            }else {
                map.put("areaCode","");
                map.put("farmName","");
            }


            endList.add(map);

        }
        return new Result(ResultCode.SUCCESS,endList);
    }

    @Override
    public Result countAll(Map<String, Object> paramsMap) {
        String farmCode = (String)paramsMap.get("farmId");
        HashMap<String,Long> endMap = new HashMap();
        Long countFarmer = baseFarmerMapper.selectCount(new QueryWrapper<BaseFarmer>().eq("farm_code", farmCode));;
        Long countPig = eartagEartagRegister2Mapper.selectCount(new QueryWrapper<EartagEartagRegister2>().eq("farm_id", farmCode).eq("live_status",1).like("earmark","31111112222"));
        Long countDevice = eartagDeviceRegisterMapper.selectCount(new QueryWrapper<EartagDeviceRegister>().eq("farm_id", farmCode));
        java.sql.Date todayDate = new java.sql.Date(new Date().getTime());
        Long countWarning = eartagAbnormalAlarmMapper.selectCount(new QueryWrapper<EartagAbnormalAlarm>().eq("farm_code",farmCode).eq("alarm_type","2").eq("add_date",todayDate));


        endMap.put("countFarmer",countFarmer);
        endMap.put("countPig",countPig);
        endMap.put("countDevice",countDevice);
        endMap.put("countWarning",countWarning);
        return new Result(ResultCode.SUCCESS,endMap);
    }

    @Override
    public Result pigMonitor(Map<String, Object> paramsMap) {
        String farmCode = (String)paramsMap.get("farmId");
        List<EartagEartagRegister2> eartagEartagRegister2s = eartagEartagRegister2Mapper.selectList(new QueryWrapper<EartagEartagRegister2>().eq("farm_id", farmCode).orderByDesc("last_time").last("limit 5"));
        return new Result(ResultCode.SUCCESS,eartagEartagRegister2s);
    }

    @Override
    public Result getFarmerAndPig(Map<String, Object> paramsMap) {
//        List<BaseFarmer> farmers = baseFargetAssetStatisticsmerMapper.selectList(new QueryWrapper<BaseFarmer>().eq("farm_code", "330727001"));
        List<BaseFarmer> farmers = baseFarmerMapper.selectList(new QueryWrapper<BaseFarmer>().eq("farm_code", "330727001"));

        List endList = new ArrayList();
        for (BaseFarmer farmer : farmers) {
            Map map = new HashMap();
//            List<EartagDeviceRegister> eartagDeviceRegisters = eartagDeviceRegisterMapper.selectList(new QueryWrapper<EartagDeviceRegister>().eq("farmer_id", farmer.getId()).eq("farm_id","330727001"));
            List<EartagDeviceRegister> eartagDeviceRegisters = eartagDeviceRegisterMapper.selectList(new QueryWrapper<EartagDeviceRegister>().eq("device_code","330112002000001"));
            map.put( "data",new ArrayList<>());
            map.put("id",farmer.getId());
            map.put("farmer",farmer.getFarmer());
            map.put("phone",farmer.getPhone());
            map.put("address",farmer.getAddress());
            map.put("farmCode",farmer.getFarmCode());
            if (ObjectUtil.isNotEmpty(eartagDeviceRegisters)){
                List farmerIds = new ArrayList();
                for (EartagDeviceRegister eartagDeviceRegister : eartagDeviceRegisters) {
                    farmerIds.add(eartagDeviceRegister.getDeviceCode());
                }
                map.put("data", eartagEartagRegister2Mapper.selectList(new QueryWrapper<EartagEartagRegister2>().in("last_device",farmerIds)));
            }
            endList.add(map);
        }

        return new Result(ResultCode.SUCCESS,endList);
    }


}
